USE [ContosoDW]
GO

/****** Object:  View [Analytics].[Sales - Complete]    Script Date: 5/6/2018 7:15:10 PM ******/
SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO


CREATE SCHEMA [Analytics]
GO


CREATE VIEW [Analytics].[Date]
AS
WITH    fiscalCalendar
          AS ( SELECT   Datekey,
                        FullDateLabel,
                        DateDescription,
                        CalendarYear,
                        CalendarYearLabel,
                        CalendarHalfYear,
                        CalendarHalfYearLabel,
                        CalendarQuarter,
                        CalendarQuarterLabel,
                        CalendarMonth,
                        CalendarMonthLabel,
                        CalendarWeek,
                        CalendarWeekLabel,
                        CalendarDayOfWeek,
                        CalendarDayOfWeekLabel,
                        FiscalYear = [CalendarYear] + CASE WHEN MONTH(Datekey) > 6 THEN 1
                                                           ELSE 0
                                                      END,
                        FiscalQuarter = ( ( MONTH(Datekey) - 1 ) / 3 + 2 ) % 4 + 1,
                        FiscalMonthNumber = ( MONTH(Datekey) + 5 ) % 12 + 1,
                        IsWorkDay,
                        IsHoliday,
                        HolidayName,
                        EuropeSeason,
                        NorthAmericaSeason,
                        AsiaSeason
               FROM     dbo.DimDate
             )
    SELECT  [Date] = CAST ([Datekey] AS DATE),
            [DateKey] = YEAR(Datekey) * 10000 + MONTH(Datekey) * 100 + DAY(Datekey),
            [Calendar Year Number] = [CalendarYear],
            [Calendar Year] = 'CY ' + CAST ([CalendarYear] AS VARCHAR(4)),
            [Calendar Year Quarter Number] = [CalendarQuarter],
            [Calendar Year Quarter] = [CalendarQuarterLabel] + '-' + CAST ([CalendarYear] AS VARCHAR(4)),
            [Calendar Year Month Number] = [CalendarMonth],
            [Calendar Year Month] = [CalendarMonthLabel] + ' ' + CAST ([CalendarYear] AS VARCHAR(4)),
            [Month Number] = DATEPART(MONTH, [Datekey]),
            [Month] = [CalendarMonthLabel],
            [Day of Week Number] = DATEPART(dw, [Datekey]),
            [Day of Week] = [CalendarDayOfWeekLabel],
            [Fiscal Year Number] = [FiscalYear],
            [Fiscal Year] = 'FY ' + CAST ([FiscalYear] AS VARCHAR(4)),
            [Fiscal Quarter Number] = [FiscalQuarter],
            [Fiscal Quarter] = 'Q' + CAST([FiscalQuarter] AS CHAR(1)),
            [Fiscal Year Quarter Number] = [FiscalYear] * 10 + [FiscalQuarter],
            [Fiscal Year Quarter] = 'Q' + CAST([FiscalQuarter] AS CHAR(1)) + '-' + CAST ([FiscalYear] AS VARCHAR(4)),
            [Fiscal Month Number] = [FiscalMonthNumber],
            [Fiscal Month] = [CalendarMonthLabel],
            [Working Day] = [IsWorkDay],
            [Is Holiday] = [IsHoliday],
            [Holiday Name] = [HolidayName],
            [Europe Season] = [EuropeSeason],
            [North America Season] = [NorthAmericaSeason],
            [Asia Season] = [AsiaSeason]
    FROM    fiscalCalendar


GO



CREATE VIEW [Analytics].[Sales - Complete]
AS
WITH    s AS ( SELECT   OnlineSalesKey,
                        StoreKey,
                        ProductKey,
                        PromotionKey,
                        CurrencyKey,
                        CustomerKey,
                        [Order Date] = CAST (DateKey AS DATE),
                        [Due Date] = CAST (DateKey + ( ProductKey + OnlineSalesKey + CustomerKey ) % 8 + 6 AS DATE),
                        -- Delivery improves over time
                        [Delivery Date] = CAST (DateKey + ( StoreKey + ProductKey + OnlineSalesKey + CustomerKey )
                        % ( 2014 - YEAR(DateKey) ) + 6 AS DATE),
                        [Order Number] = SalesOrderNumber,
                        [Order Line Number] = SalesOrderLineNumber,
        -- Increases the quantity to 20% of the sales transactions
                        [Quantity] = SalesQuantity
                        * CASE WHEN SalesQuantity = 1
                                    AND ( ProductKey + OnlineSalesKey + CustomerKey ) % 5 = 0
                               THEN ( ProductKey + StoreKey + ( OnlineSalesKey * 4 ) + CustomerKey ) % 3 + 2
                               ELSE SalesQuantity
                          END,
                        [Unit Price] = UnitPrice,
                        [Unit Discount] = DiscountAmount,
                        [Unit Cost] = UnitCost,
                        [Net Price] = UnitPrice - DiscountAmount
               FROM     dbo.FactOnlineSales
               WHERE    SalesQuantity >= 1 -- Ignore Returned items
             )
    SELECT  s.OnlineSalesKey,
            s.StoreKey,
            s.ProductKey,
            s.PromotionKey,
            s.CurrencyKey,
            s.CustomerKey,
            [OrderDateKey] = YEAR(s.[Order Date]) * 10000 + MONTH(s.[Order Date]) * 100 + DAY(s.[Order Date]),
            [DueDateKey] = YEAR(s.[Due Date]) * 10000 + MONTH(s.[Due Date]) * 100 + DAY(s.[Due Date]),
            [DeliveryDateKey] = YEAR(s.[Delivery Date]) * 10000 + MONTH(s.[Delivery Date]) * 100 + DAY(s.[Delivery Date]),
            s.[Order Date],
            s.[Due Date],
            s.[Delivery Date],
            s.[Order Number],
            s.[Order Line Number],
            s.Quantity,
            s.[Unit Price],
            s.[Unit Discount],
            s.[Unit Cost],
            s.[Net Price]
       -- The total line values are not included in the view; they should be calculated in DAX measures instead
       --[Sales Amount] = s.Quantity * s.[Net Price],
       --[Discount Amount] = s.Quantity * s.[Unit Discount],
       --[Total Cost] = s.Quantity * s.[Unit Cost]
    FROM    s



GO


